/** *************************************************************************
 **
 ** Copyright (C) 2008 Heinz van Saanen
 **
 ** This file is part of the function viewer Zhu3D.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 3 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
****************************************************************************/

/** Includes */
#include <QApplication>
#include <QIcon>
#include <QMessageBox>
#include <QObject>
#include <QTranslator>
#include <QtGlobal>

#include "debug.h"
#include "property.h"
#include "mainwindow.h"
#include "glwidget.h"
#include "ligedit.h"
#include "matedit.h"
#include "entedit.h"
#include "aniedit.h"
#include "moredit.h"
#include "picedit.h"
#include "fntedit.h"
#include "demedit.h"
#include "funedit.h"
#include "speedit.h"
#include "diredit.h"
#include "usredit.h"
#include "legedit.h"
#include "sysinfo.h"
#include "cpuinfo.h"


/** Miscellaneous globals */
QString appPath;		// Path where Zhu3D was started from
QString argStr("");		// Holds arguments from bash or window-manager
float MHz=0.0f;			// Stays zero, when no supported hw-timer is found


/** Denotes the number of existing CPU-cores. The file property.cpp provides
 ** numCPUset, which holds the core number according to user settings */
int numCPU;


/** Global widgets */
MaiWindow *maiWin;		// Global because the system-font-stuff needs it
OGLWidget *oglWid;		// OpenGL viewer
usrWidget *usrEdi;		// User defined functions/constants editor
ligWidget *ligEdi;		// Light editor
matWidget *matEdi;		// Material editor
entWidget *entEdi;		// Entire editor
aniWidget *aniEdi;		// Animation editor
morWidget *morEdi;		// Morphing editor
picWidget *picEdi;		// Pic-dimension editor
fntWidget *fntEdi;		// Fontsize editor
demWidget *demEdi;		// Demo mode
funWidget *funEdi;		// Function/parameter editor
speWidget *speEdi;		// Special settings
dirWidget *dirEdi;		// Directory settings
legWidget *legEdi;		// Legends editor
#ifdef ZHUDEBUG
errWidget *errEdi;		// Error messages
#endif


/** Localization-infos loading help-files and other stuff
 ** Used extern in modules fntedit, mainwindow and usredit */
QString zhuLocale;
QTranslator zhuTranslator;


/** Install own message handler
 ** Normal stderr and Qt4-messages are redirected to own error window */
#ifdef ZHUDEBUG
void myMessageHandler(QtMsgType type, const char *msg) {
	switch (type) {
		case QtDebugMsg:
			PRINT((QString)"Debug: "+msg);
			break;
		case QtWarningMsg:
			PRINT((QString)"Warning: "+msg);
			break;
		case QtCriticalMsg:
			PRINT((QString)"Critical: "+msg);
			break;
		case QtFatalMsg:
			PRINT((QString)"Fatal: "+msg);
			break;
		abort();
	}
}
#endif


/** *************************************************************************
 ** USAGE: zhu3d [foozhufile]
****************************************************************************/
int main(int argc, char *argv[]) {

	/** Very basic initializations */
	#ifdef ZHUDEBUG
	qInstallMsgHandler(myMessageHandler);
	#endif
	QApplication app(argc, argv);

	/** Very first, maybe needed from others */
	#ifdef ZHUDEBUG
	errEdi = new errWidget;
	errEdi->show();
	#endif

	/** Detect the number of real CPU-cores
	 ** Default numCPUset=numCPU, what may be altered from module fntEdi later */
	numCPU=CPUINFO::cores();
	if(numCPU<1 || numCPU>MAXCPU)
		numCPU=1;
	numCPUset=numCPU;

	/** Check clock frequency. Hardware dependent CPU timer has to be implemented in
	 ** ticks.h for this. If not, this does no harm and frequency is just "unknown"
	 ** For Windows this is disabled automatically because of bad timer-granularity */
	#if (defined(__i386__) || defined(__x86_64__) || \
		defined(__powerpc__) || \
		(defined(__mips__) && (defined(MIPSEL) || defined (__MIPSEL__))) )
		MHz=CPUINFO::MHz();
	#endif

	/** Initialize logo and application-path */
	QIcon appIcon(":/images/icon.png");
	app.setWindowIcon(appIcon);
	appPath=QCoreApplication::applicationDirPath();

	/** General internationalization stuff */
	QTranslator qtTranslator;
	qtTranslator.load("qt_"+zhuLocale);
	app.installTranslator(&qtTranslator);

	/** Set Zhu3D languages */
	zhuLocale=QLocale::system().name();
	zhuLocale.resize(zhuLocale.lastIndexOf('_'));
	zhuTranslator.load(MYLANDIR+"zhu3d_"+zhuLocale);
	app.installTranslator(&zhuTranslator);

	/** Patch for Qt 4.0.0 (and 4.5.beta) to overcome a locale error
	 ** Set LC_NUMERIC to force using the dot as decimal delimiter */
	#if (QT_VERSION == 0x040000) || (QT_VERSION == 0x040500)
	setlocale(LC_NUMERIC, "C");
	#endif

	/** Check for argument from command line */
	if(argc==2)
		argStr=argv[1];

	/** Some more basic initializations */
	initSysInfo();
	fpError::initParserQt();
	if(!initAllParsers()) {
		QMessageBox::critical(
					0,
					QObject::tr("Program start")+"\n",
					fpError::getQtMessage(11)
		);
		return -1;
	}

	/** OpenGL defaults. Enable motion blur through accumulation buffer */
	QGLFormat fmt;
	fmt.setAccum(true);
	fmt.setAlpha(true);
	// Multisampling is costly and mostly not worth the effort. So disabled as default
	// fmt.setSampleBuffers(true);
	if(!fmt.hasOpenGL()) {
		QMessageBox::critical(
					0,
					QObject::tr("Program start")+"\n",
					QObject::tr("Missing OpenGL! Terminating now")
		);
		return -1;
	}
	QGLFormat::setDefaultFormat(fmt);

	/** Create globals */
	oglWid = new OGLWidget;		// OpenGL viewer first, some inits done here!
	usrEdi = new usrWidget;		// User defined functions editor next! Needed from fntEdi later
	ligEdi = new ligWidget;		// Light editor
	matEdi = new matWidget;		// Material editor
	entEdi = new entWidget;		// Entire editor
	aniEdi = new aniWidget;		// Animation editor
	morEdi = new morWidget;		// Morphing editor
	picEdi = new picWidget;		// Pic-dimension editor
	funEdi = new funWidget;		// Function editor
	fntEdi = new fntWidget;		// Appeareance settings
	demEdi = new demWidget;		// Demo mode
	speEdi = new speWidget;		// Specials settings
	dirEdi = new dirWidget;		// Directory settings
	legEdi = new legWidget;		// Legends editor

	/** Create and bring up the mainwindow finally */
	maiWin = new MaiWindow;
	maiWin->show();

	return app.exec();
}
