/** *************************************************************************
 **
 ** Copyright (C) 2008 Heinz van Saanen
 **
 ** This file is part of the function viewer Zhu3D.
 **
 ** This file may be used under the terms of the GNU General Public
 ** License version 3 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** *************************************************************************/

#ifndef _TSC_H_
#define _TSC_H_


/** Taming the monster a little bit:-) */
typedef unsigned long long TSC_UINT;


/** *************************************************************************
 ** CPU-timestamp counters implemented for: x86, x86_64, PowerPC and Mips
 ** Calibrates itself with an correction factor when instanciated
 ** Corrects negative resuls which could occure through optimizing/caching
 ** For maximum precision the x86-stuff serializes instructions additionally
 **
 ** Compilers proofed so far:
 **
 ** x86_64 and *nix  :  gcc 4.3.1, icc 10.1
 ** x86 and *nix     :  gcc 4.0.2
 ** x86 and Windows  :  MinGW 3.4.2
 **
 ** No idea about Mips/PowerPC. Surely a minimum of some 35.000 copies are out
 ** there and there never was any complaint from distributors/users :-)
 ** *************************************************************************/
class TSC {

public:
	TSC() { calc_min_ovh(); start(); }
	~TSC() {}

	/** Start/stop functions */
	void start() { tsc=fetch(); }
	TSC_UINT stop() { if((tsc=fetch()-tsc)>ovh) return tsc-=ovh; return tsc=0ULL; }

	/** Return correction-cycles for conveniance */
	TSC_UINT min_ovh() { return ovh; }

private:
	TSC_UINT tsc, ovh;
	inline TSC_UINT fetch();

	/** Find minimum tsc-overhead */
	void calc_min_ovh() {
		ovh = 1<<24;
		for(long i=0; i<8; ++i) {
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
			tsc=fetch(); tsc=fetch()-tsc; if(tsc<ovh) ovh=tsc;
		}
	}
};


/** Hardware dependent CPU-cycle-counters
 ** *************************************************************************/

/** x86 or x86_64 CPU's */
#if (defined(__i386__) || defined(__x86_64__))
inline TSC_UINT TSC::fetch() {

	#if defined(_LP64)
		unsigned int res[2];
	#else
		volatile unsigned int res[2];
	#endif

	#if (defined(__GNUC__) && !defined(__INTEL_COMPILER))

	/**  GCC 64 bit mode on *nix-systems */
	#if defined(_LP64)
		__asm__ __volatile__ (				// Save rbx and serialize
			"pushq %%rbx \n"
			"xorl %%eax,%%eax \n"
			"cpuid \n"
			::: "%rax", "%rcx", "%rdx");

		__asm__ __volatile__ (				// Read TSC. Store edx:eax in res
			"rdtsc \n"
			: "=a" (res[0]), "=d" (res[1]));

		__asm__ __volatile__ (				// Serialize again and restore rbx
			"xorl %%eax,%%eax \n"
			"cpuid \n"
			"popq %%rbx \n"
			::: "%rax", "%rcx", "%rdx");

	/**  GCC 32bit mode on *nix-systems or MinGW32 */
	#else
		__asm__ __volatile__ (				// Save ebx and serialize
			"pushl %%ebx \n"
			"xorl %%eax,%%eax \n"

			"cpuid \n"
			::: "%eax", "%ecx", "%edx");

		__asm__ __volatile__ (				// Read TSC. Store edx:eax in res
			"rdtsc \n"
			: "=a" (res[0]), "=d" (res[1]) );

		__asm__ __volatile__ (				// Serialize again and restore ebx
			"xorl %%eax,%%eax \n"
			"cpuid \n"
			"popl %%ebx \n"
			::: "%eax", "%ecx", "%edx");
	#endif

	/** Else must be ICC. (Open question: Is MSC working too?) */
	#else
		__asm {
			xor eax, eax
			cpuid							// Serialize
			rdtsc							// Read TSC
			mov dword ptr res, eax			// Store low dword in res[0]
			mov dword ptr res+4, edx		// Store high dword in res[1]
			xor eax, eax
			cpuid							// Serialize again
		}
	#endif

	/** Return tsc for 32/64 bit *nix or Win */
	return (((TSC_UINT)res[1] << 32) | (res[0]));
}


/** Check for Mips CPU's. Compiler = gcc */
#elif (defined(__mips__) && (defined(MIPSEL) || defined (__MIPSEL__)))
inline TSC_UINT TSC::fetch() {
	TSC_UINT tmp;
	__asm__ __volatile__("mfc0 %0,$9; nop" : "=r" (tmp));
	return tmp;
}


/** Check for PowerPC CPU's. Compiler = gcc */
#elif defined(__powerpc__)
inline TSC_UINT TSC::fetch(void) {
	unsigned int high, low, tmp;
	__asm__ volatile(
		"loop:                  \n"
		"\tmftbu   %0           \n"
		"\tmftb    %1           \n"
		"\tmftbu   %2           \n"
		"\tcmpw    %2,%0        \n"
		"\tbne     loop         \n"
		: "=r"(high),"=r"(low),"=r"(tmp)
	);
	return ((TSC_UINT)high << 32) | low;
}
#endif


/** _TSC_H_ */
#endif
